" eddie.vim
"
" some VIM gizmos to make us ex-Eddie users feel a little more
" at home.
"
" version 0.1.1
"
" by dsandler@dsandler.org
" http://dsandler.org/soft/vim

if exists("loaded_eddie") || &cp
	finish
endif
let loaded_eddie = 1

" ----------------------------------------------------------------------
" [1] Configuration.

" Location of the presto.py Python script.  Leave blank if you don't have
" Python either in vim or on your system, or if you don't want to use presto.
"let s:presto_script = expand($HOME) . '/.vim/presto.py'

if has('win32')
	let s:plugins = expand("$VIM") . '\vimfiles\plugin'
	let s:slash = '\'
elseif has('unix')
	let s:plugins = expand("$HOME") . '/.vim/plugin'
	let s:slash = '/'
elseif has('gui_mac')
	let s:plugins = expand("$VIM") . ':vimfiles:plugin'
	let s:slash = ':'
else
	let s:plugins = expand("$VIM")
	let s:slash = '/'
endif

let s:presto_script = s:plugins . s:slash . 'eddie' . s:slash . 'presto.py'

" Location of Python.  If you don't have it, don't worry about it.
if !exists("g:python_exe") && !has('python')
	if has('win32')
		let g:python_exe = 'c:\python22\python.exe'
	elseif has('unix')
		let g:python_exe = "python"
	endif
endif

" Patterns that should be matched by the ToDo plugin.
let s:todo_grep_pat = '(TODO\|XXX\|FIX\|FIXME):'
let s:todo_findstr_pat = 'TODO: XXX: FIX: FIXME:'

" Status line, with extra Eddie-like goodies.
function! VarExists(var, val)
	if exists(a:var) | return a:val | else | return '' | endif
endfunction
set statusline=(%-n)\ %<%f%m\ %r%=%{VarExists('g:presto_charged','<presto\!>\ ')}%y\ %l,%c%V\ %P


" ----------------------------------------------------------------------
" [2] Keybindings.

" ^T: Twiddle chars at point.  [Note: this will overwrite existing ^t]
nmap <C-T> h"tx"tp
imap <C-T> <Esc>"tx"tpi

" Open file, with syntax <FileName>:<LineNo>.
" CURRENTLY BROKEN
"noremap gf :GotoFileLine <C-R><C-F><CR>
"noremap gF :SplitGotoFileLine <C-R><C-F><CR>
"vnoremap gF y:SplitGotoFileLine <C-R>"<CR>
"if has("gui_mac")
"	noremap <D-d> :SplitGotoFileLine <C-R><C-F><CR>
"endif

" Block-move.
if has("gui_mac")
	map <silent> <unique> <C-D-Up> :MoveLinesUp<CR>
	map <silent> <unique> <C-D-Down> :MoveLinesDown<CR>
	map <silent> <unique> <D-[> :MoveLinesLeft<CR>
	map <silent> <unique> <D-]> :MoveLinesRight<CR>
else
	map <silent> <unique> <C-A-Up> :MoveLinesUp<CR>
	map <silent> <unique> <C-A-Down> :MoveLinesDown<CR>
	map <silent> <unique> <A-[> :MoveLinesLeft<CR>
	map <silent> <unique> <A-]> :MoveLinesRight<CR>
	" alternate bindings for those who don't have C-A available
	map <silent> <unique> <C-S-Up> :MoveLinesUp<CR>
	map <silent> <unique> <C-S-Down> :MoveLinesDown<CR>
endif

" General keyboard navigation.
if has("gui_mac")
	map <silent> <unique> <D-Up> :1<CR>
	map <silent> <unique> <D-Down> :$<CR>
	map <silent> <unique> <D-Left> ^
	map <silent> <unique> <D-Right> $
else
	map <silent> <unique> <A-Up> :1<CR>
	map <silent> <unique> <A-Down> :$<CR>
	map <silent> <unique> <A-Left> ^
	map <silent> <unique> <A-Right> $
endif

" ** Plugin-like features
" The following are functions reminiscent of the Eddie plugins.  I've grouped
" them all under the \ macro prefix; for example, the ToDo plugin is accessed
" by pressing '\', then 't'.  (Note that you can change the choice of
" this leading prefix by reletting the variable 'mapleader'.)

" Presto, the magic prototyper.
map <silent> <unique> <Leader>p :PrestoToggle<CR>
	" [I provide an alternative syntax, since the original magic prototyper
	" supported it: alt-P.  (Control-P is a well-known vi command, so I
	" didn't want to clobber it.)]
	if has('gui_mac')
		map <silent> <unique>  :PrestoToggle<CR> | " sorry about the 8-bit character!
	else
		map <silent> <unique> <A-P> :PrestoToggle<CR>
	endif

" ToDo.  
map <silent> <unique> <Leader>t :ToDo<CR>

" ----------------------------------------------------------------------
" [3] Support functions & macros.

" utility
function! s:Min(a,b)
	return((a:a > a:b) ? a:b : a:a)
endfunction
function! s:Max(a,b)
	return((a:b < a:a) ? a:a : a:b)
endfunction


" Open file under cursor at line.
" ex: FooFile.c:10:
function! s:GotoFileLine(filespec)
	let s:fn = substitute(a:filespec, ":\\d*$", "", "")
	let s:ln = substitute(a:filespec, "^.*:\\(\\d*\\)$", "\\1", "")
	if s:ln == ""
		exe "edit " . a:filespec
	else
		exe "edit +" . s:ln . " " . s:fn
	endif
endfunction
command! -nargs=1 -complete=file GotoFileLine call GotoFileLine(<f-args>)
function! s:SplitGotoFileLine(filespec)
	let s:fn = substitute(a:filespec, ':\d\+:\?$', "", "")
	let s:ln = match(a:filespec, ':\(\d\+\):\?$', "")
	exe "wincmd w"
	if s:fn != ""
		if s:ln == "-1"
			echo "edit " . a:filespec
		else
			echo "edit +" . s:ln . " " . s:fn
		endif
	endif
endfunction
" function! SplitGotoFileLineRange() range
" 	let first = s:Min(a:firstline, a:lastline)
" 	let last  = s:Max(a:firstline, a:lastline)
" 	call SplitGotoFileLine(fs)
" endfunction
command! -nargs=1 -complete=file SplitGotoFileLine call SplitGotoFileLine(<f-args>)
"noremap gF :call GotoFileLine("<C-R><C-F>")<CR>


" Line-move.

function! s:MoveLinesUp() range
	let s:first = s:Min(a:firstline, a:lastline)
	let s:last  = s:Max(a:firstline, a:lastline)
	if 1 != s:first
		execute s:first . ',' . s:last . 'm' . (s:first - 2)
		if (s:last-s:first >= 1)
			exe 'normal V' . (s:last-s:first) . 'k'
		else
			exe 'normal V'
		endif
	endif
endfunction
command! -nargs=0 MoveLineUp .,.call <SID>MoveLinesUp()
command! -range -nargs=0 MoveLinesUp <line1>,<line2>call <SID>MoveLinesUp()

function! s:MoveLinesDown() range
	let s:first = s:Min(a:firstline, a:lastline)
	let s:last  = s:Max(a:firstline, a:lastline)
	if line("$") != s:last
		execute s:first . ',' . s:last . 'm' . (s:last + 1)
		if (s:last-s:first >= 1)
			exe 'normal V' . (s:last-s:first) . 'k'
		else
			exe 'normal V'
		endif
	endif
endfunction
command! -nargs=0 MoveLineDown .,.call <SID>MoveLinesDown()
command! -range -nargs=0 MoveLinesDown <line1>,<line2>call <SID>MoveLinesDown()

function! s:MoveLinesLeft() range
	let s:first = s:Min(a:firstline, a:lastline)
	let s:last  = s:Max(a:firstline, a:lastline)
	execute s:first . ',' . s:last . '<'
	if (s:last-s:first >= 1)
		exe 'normal V' . (s:last-s:first) . 'k'
	else
		exe 'normal V'
	endif
endfunction
command! -nargs=0 MoveLinesLeft .,.call <SID>MoveLinesLeft()
command! -range -nargs=0 MoveLinesLeft <line1>,<line2>call <SID>MoveLinesLeft()

function! s:MoveLinesRight() range
	let s:first = s:Min(a:firstline, a:lastline)
	let s:last  = s:Max(a:firstline, a:lastline)
	execute s:first . ',' . s:last . '>'
	if (s:last-s:first >= 1)
		exe 'normal V' . (s:last-s:first) . 'k'
	else
		exe 'normal V'
	endif
endfunction
command! -nargs=0 MoveLinesRight .,.call <SID>MoveLinesRight()
command! -range -nargs=0 MoveLinesRight <line1>,<line2>call <SID>MoveLinesRight()


" Lines.  Why?

command! -range -nargs=0 Lines :echo <line2> - <line1> + 1 "lines selected"


" Presto, the magic prototyper.

function! s:Presto() range
	if exists("s:presto_script") == 0
		echohl ErrorMsg
		echo 'presto: presto not available'
		echohl None
		finish
	endif

	let s:first = s:Min(a:firstline, a:lastline)
	let s:last  = s:Max(a:firstline, a:lastline)
	
	unlet! g:presto_charged
	
	let s:tmpfile = tempname()
	let g:presto_charged = tempname()

	if has('python')
		silent exe s:first . ',' . s:last . 'pyfile "' . s:presto_script . '"'
	elseif exists("g:python_exe")
		silent exe s:first . ',' . s:last . 'write ' . s:tmpfile
		let s:cmd = g:python_exe . ' "' . s:presto_script . '" ' .  s:tmpfile
		"let s:cmd = (s:cmd . " > " . g:presto_charged)
		let s:p_save = @p
		let @p = system(s:cmd)
		if @p != ""
			silent exe ":redir >" . g:presto_charged
			silent echo @p
			silent redir END
		endif
		let @p = s:p_save
		call delete(s:tmpfile)
	else
		echohl ErrorMsg
		echo 'presto: python not available'
		echohl None
		return
	endif
	
	echohl WarningMsg
	if exists("g:presto_charged")
		if ! filereadable(g:presto_charged)
			unlet! g:presto_charged
		elseif getfsize(g:presto_charged) == 0
			call delete(g:presto_charged)
			unlet! g:presto_charged
		endif
	endif
	if ! exists("g:presto_charged")
		echo 'presto: no C/C++ function signatures detected in selection or line'
	else
		echo 'presto: C/C++ magic prototypes are charged'
	endif
	echohl None
endfunction

function! s:PrestoToggle() range
	if ! exists("g:presto_charged")
		exe a:firstline.','.a:lastline . 'call <SID>Presto()'
	else
		silent exe ':r ' . g:presto_charged
		echohl WarningMsg 
		echo 'presto: C/C++ magic prototypes discharged at cursor' 
		echohl None
		call delete(g:presto_charged)
		unlet! g:presto_charged
	endif
endfunction

command! -range -nargs=0 Presto <line1>,<line2>call <SID>Presto()
command! -nargs=0 PrestoToggle .,.call <SID>PrestoToggle()
command! -range -nargs=0 PrestoToggle <line1>,<line2>call <SID>PrestoToggle()

" ToDo plugin.  Originally by Alan Ellis.  Now reduced to a scant few lines in
" vim.
function! s:ToDo()
	if has('win32')
		" note extra junk at the end to force windows to print the filename
		silent exe ':grep /I "' . s:todo_findstr_pat . '" ' . "% " .  '""' 
	else
		silent exe ":grep -EinH -e '" . s:todo_grep_pat . "' %"
	endif
	copen 5
endfunction
command! -nargs=0 ToDo call <SID>ToDo()


" -end-
