#!/usr/bin/python
#
# the midnight tokenizer
#

import sys, re, string
from cStringIO import StringIO

try:
	import vim
except:
	vim = None

class midnight:
	def __init__(_): 
		pass

	cpp_keywords = [
		'if', 'else', 'switch', 'case', 'return', 'while', 'for', 'do', 'class',
		'namespace', 'using', 'struct', 'typedef', 'public:', 'private:',
		'protected:', 'virtual',
	]

	cpp_grammar = [
		['ws', r'(?m)[ \t\r\n]+'],
		['keyword',
			r'(?:' + string.join(cpp_keywords,"|") + r')'],
		['name', r'[A-Za-z_][A-Za-z0-9_]*'],
		['string', r'"(?:\\"|[^"])*"'],
		['lparen', r'\('],
		['rparen', r'\)'],
		['lbrace', r'\{'],
		['rbrace', r'\}'],
		['semicolon', ';'],
		['scope', '::'],
		['comma', ','],
		['operator', r'(?:\*|\+|-\>?|%|&&?|\|\|?|=|&|\<|\>)'],
		['linecomment', r'//[^\n]*\n'],
		['blockcomment', r'/\*(?:[^\*]|\*[^/])*\*/\s*'], # note extra ws at end
		['preproc', r'^#[^\n]*\n'],
		['unknown-word', r'\S*'],
	]

	class tokenizer:
		class token:
			def __init__(_, type, data):
				_.type = type ; _.data = data
			def __repr__(_):
				s = "<token: " + _.type
				if _.data:
					s = s + " " + `_.data`
				return s + ">"

		def __init__(_, gram):
			_.tokens = []
			_.grammar = gram
		
		def reset(_):
			_.tokens = []
		def set_grammar(_,g):
			_.grammar = g

		def push(_, token):
			_.tokens.append(token)

		def pop(_):
			if len(_.tokens) == 0: return None
			t = _.tokens[0]
			del _.tokens[0:1]
			return t

		def more(_):
			return len(_.tokens) > 0
		
		def peek(_):
			return _.tokens[0]

		def scan(_, stream):
			#for line in xreadlines.xreadlines(stream):
			#	if line == None: break
			line = stream.read()
			while line != "":
				#print "\033[1manalyzing:\033[0m " + `line`
				grabbedtoken = None
				for (tokentype, pattern) in _.grammar:
					#print "\033[7mtest:\033[0m " + `_.grammar[tokentype]` + "\n"
					m = re.match(pattern, line, re.DOTALL)
					if m != None:
						_.push(_.token(tokentype, m.group(0)))
						line = line[m.end(0):]
						grabbedtoken = 1
						break # out of match loop
				if not grabbedtoken:
					break
				# nothing found
					
			return _.tokens

# Presto! the magic prototyper in Python
#
# TODO:
#
#   [11:11] <void> magic-prototyper, it will also put inline identifiers if
#   you 'paste' them into the header.
#
#   [11:13] <em> also, if you feed it a block of inline methods defined in the
#   class, it hangs on to the code (though it comes out weirdly formatted)

class presto:
	DEBUG=0
	options = {
		'preserve_comments': 0
	}
	def debug(_,s):
		if _.DEBUG:
			print "    \033[0;32m" + s + "\033[0m"

	def process(_,tok,out):
		nest = []
		bracelev = 0
		funcwords = []
		args = []

		t = None
		t0 = None
		
		while 1:
			if not tok.more(): break
			
			t = tok.pop()

			if (t.type == 'keyword'):
				if (t.data == 'namespace'):
					out.write("using namespace ")
					nsparts = []
					if (tok.more()):
						while tok.more():
							t2 = tok.pop()
							if t2.type == 'name':
								break
						if t2:
							nsparts.append(t2.data)
							out.write(string.join(nsparts, "::") + ";\n\n")
							nest.append([t,string.join(nsparts,"::")])
				elif (t.data == 'class'):
					while tok.more():
						t2 = tok.pop()
						if t2.type == 'name': break
					if t2: 
						nest.append(['class',t2.data])
			elif (t.type == 'lbrace'):
				bracelev = bracelev + 1
				if len(funcwords) > 0:
					# note: we will be starting a function body, so we know the
					# thing we have to do once we hit the rbrace is do a func ->
					# proto conversion
					pass
			elif (t.type == 'rbrace'):
				bracelev = bracelev - 1
				if len(funcwords) > 0:
					if bracelev == 0:
						# just finished a complete function decl
						_.debug("[}] funcwords=" + `funcwords`)
						if len(funcwords) > 3:
							_.debug("    -> funcwords[-2] == " + `funcwords[-2]`)
							if funcwords[-2] == "::":
								del funcwords[-3:-1]
						if len(funcwords) > 1:
							out.write(string.join(funcwords[:-1]," "))
							out.write(" ")
						out.write(funcwords[-1] + "(");
						out.write(string.join(args,""))
						out.write(");\n");
						args = []
						funcwords = []
				elif len(nest)>0: 
					nest.pop()
				
			elif (t.type == 'semicolon'):
				if bracelev == len(nest): # are we outside a function?
					if len(funcwords) > 0:
						_.debug("[;] bracelev = %d, nest = %d" % (bracelev, len(nest)))
						if len(funcwords) > 1:
							out.write(string.join(funcwords[:-1]," "))
						if len(nest) > 0:
							if len(funcwords) > 1: out.write(" ")
							out.write(string.join(
									map(lambda x:x[1],
										filter(lambda x:x[0]=='class',
											nest
										)
									), "::"
								))
							out.write("::")
						elif len(funcwords) > 1: out.write(" ")
						out.write(funcwords[-1])
						out.write("(")
						out.write(string.join(args,""))
						out.write(")\n{\n\t\n}\n\n")
						funcwords = []
						args = []
			elif _.options.get("preserve_comments") \
			and (t.type == 'linecomment' or t.type == 'blockcomment'):
				if bracelev == len(nest): # not in the body of a function
					out.write(t.data + " ")
			elif (t.type == 'lparen'):
				if bracelev == len(nest): # not in the body of a function
					_.debug("[(] starting args");
					parenlev = 1
					while tok.more():
						t2 = tok.pop()
						if t2.type == 'lparen': 
							parenlev = parenlev + 1
							_.debug("    -> '(' [lev %d]" % parenlev)
						elif t2.type == 'rparen': 
							parenlev = parenlev - 1
							_.debug("    -> ')' [lev %d]" % parenlev)
							if parenlev == 0: break
						else:
							_.debug("    -> arg:" + `t2.data`);
							args.append(t2.data)
			elif (t.type == 'name'):
				if bracelev == len(nest):
					if t0 and (t0.type == 'class' or t0.type == 'namespace'):
						pass
					else:
						_.debug("[funcword] %s -- bracelev = %d, nest = %d" %
							(`t.data`,bracelev, len(nest)))
						funcwords.append(t.data)
			elif (t.type == 'scope'):
				funcwords.append(t.data)
			
			t0 = t

	def translate(_,fd,ofd):
		tok = midnight.tokenizer(midnight.cpp_grammar)
		tokens = tok.scan(fd)
		_.debug(`tokens`)
		_.process(tok,ofd)

def main():
	if vim:
		vim_input_stream = StringIO()
		range = vim.current.range
		for l in range:
			vim_input_stream.write(l + "\n")
		vim_input_stream.seek(0)
		p = presto()
		vim_output_stream = open(vim.eval("g:presto_charged"),'w')
		p.translate(vim_input_stream,vim_output_stream)
	else:
		files = sys.argv[1:]
		p = presto()
		if len(files) == 0:
			p.translate(sys.stdin,sys.stdout)
		else:
			for f in files:
				try:
					fd = open(f, 'r')
				except:
					sys.stderr.write("error: could not open file: " + f)
					sys.exit(1)
				p.translate(fd,sys.stdout)

if __name__ == "__main__":
	main()
